@@ -686,7 +686,9 @@ class FunctionInvariant(_InvariantJob):
             )
         ) or "cython_function_or_method" in str(type(function)):
             return cls.get_cython_source(function)
-        elif isinstance(function, types.MethodType) and "cython_function_or_method" in str(type(function.__func__)):
+        elif isinstance(
+            function, types.MethodType
+        ) and "cython_function_or_method" in str(type(function.__func__)):
             return cls.get_cython_source(function.__func__)
         else:
             # print(repr(function))
@@ -696,7 +698,6 @@ class FunctionInvariant(_InvariantJob):
     @classmethod
     def _get_func_hash(cls, key, function):
         if not util.global_pipegraph or key not in util.global_pipegraph.func_hashes:
-            print("_get_func_hash", type(function))
             source = inspect.getsource(function).strip()
             # cut off function definition / name, but keep parameters
             if source.startswith("def"):
@@ -718,7 +719,6 @@ class FunctionInvariant(_InvariantJob):
 
     @classmethod
     def _hash_function(cls, function):
-        print("hash_function", type(function))
         key = id(function.__code__)
         new_source, new_funchash = cls._get_func_hash(key, function)
         new_closure = cls.extract_closure(function)
@@ -737,7 +737,6 @@ class FunctionInvariant(_InvariantJob):
         ):
 
             return self._get_invariant_from_non_python_function(self.function)
-        print("get_invariant", type(self.function))
         new_source, new_funchash, new_closure = self._hash_function(self.function)
         return self._compare_new_and_old(new_source, new_funchash, new_closure, old)
 
@@ -1331,6 +1330,8 @@ class MultiFileGeneratingJob(FileGeneratingJob):
         """If @rename_broken is set, any eventual outputfile that exists
         when the job crashes will be renamed to output_filename + '.broken'
         (overwriting whatever was there before)
+
+        @empty_ok may be a bool or a dict (filenames: str -> bool)
         """
         # filenames = sorted([verify_job_id(f) for f in filenames])
         self.verify_arguments(function, rename_broken, empty_ok)
@@ -1343,6 +1344,11 @@ class MultiFileGeneratingJob(FileGeneratingJob):
         self.rename_broken = rename_broken
         self.do_ignore_code_changes = False
         self.empty_ok = empty_ok
+        if isinstance(self.empty_ok, dict):
+            if set(self.filenames) != set(self.empty_ok):
+                raise ValueError(
+                    "empty_ok had a different set of filenames than were passed to the MultiFileGeneratingJob"
+                )
 
     def verify_arguments(self, function, rename_broken, empty_ok):
         if not hasattr(function, "__call__"):
@@ -1363,7 +1369,11 @@ class MultiFileGeneratingJob(FileGeneratingJob):
 
     def calc_is_done(self, depth=0):
         for fn in self.filenames:
-            if self.empty_ok:
+            if (
+                self.empty_ok is True
+                or isinstance(self.empty_ok, dict)
+                and self.empty_ok[fn]
+            ):
                 if not util.file_exists(fn):
                     return False
             else:
@@ -1404,13 +1414,18 @@ class MultiFileGeneratingJob(FileGeneratingJob):
             six.reraise(*exc_info)
         self._is_done = None
         missing_files = []
-        if self.empty_ok:
-            filecheck = util.file_exists
-        else:
-            filecheck = util.output_file_exists
-        for f in self.filenames:
-            if not filecheck(f):
-                missing_files.append(f)
+        for fn in self.filenames:
+            if (
+                self.empty_ok is True
+                or isinstance(self.empty_ok, dict)
+                and self.empty_ok[fn]
+            ):
+                filecheck = util.file_exists
+            else:
+                filecheck = util.output_file_exists
+            if not filecheck(fn):
+                missing_files.append(fn)
+
         if missing_files:
             raise ppg_exceptions.JobContractError(
                 "%s did not create all of its files.\nMissing were:\n %s"
